//====================================================== file = in_sel.c ======
//=  Oldest cell first (OCF) arbiter for Saad PPS switch simulation model     =
//=============================================================================
//=  Notes:                                                                   =
//=        1) Models oldest cell first algorithm                              = 
//-----------------------------------------------------------------------------
//=  Contains: in_sel() 
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//-----------------------------------------------------------------------------
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files --------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

void in_sel(void)
{
 int   i,j,l;  // Generic loop counters
 long  tag;    // Arrival timestamp of HOL cell in inport buffers 
 int   accept[N][N]; // Selected inport-outport pair for cell forwarding
 int   flag1[K][N][N],flag2[N][N]; // Boolean flags
 int   minimum; // For computing minimum timestamp among the HOL cells 
 int   min_inport,min_outport; // Inport,outport of cell with minimum timestamp
 int   channel_num; // Switch plane number
 int   flag_inport[N],flag_outport[N]; // Boolean flags
 int   first_flag;  // Boolean flag
 int   temp1[K][N][N],temp2[N][N],temp3[N][N];  // Temporary variables 

 create("in_sel");

 // Initialization the variables
 for(i=0;i<N;i++)
 {
   for(j=0;j<N;j++)
   {
     accept[i][j] = FALSE;
     flag2[i][j] = FALSE;

     for(l=0;l<K;l++)
     {
       flag1[l][i][j] = FALSE;
     }
   }
 }

 for(i=0;i<N;i++)
 {
   flag_inport[i] = FALSE;
   flag_outport[i] = FALSE;
 }

 // Infinite Loop
 while(TRUE)
 {

   // Hold for synchronization
   hold(0.0);

   // Getting timestamps of all HOL cells in K switch planes
   for(l=0;l<K;l++)
   {
     for(i=0;i<N;i++)
     {
       for(j=0;j<N;j++)
       {

         if((flag1[l][i][j]==FALSE)&&(msg_cnt(Tag_number[l][i][j])>0))
         {
           receive(Tag_number[l][i][j], (long *) &tag);
           temp1[l][i][j] = tag;
           flag1[l][i][j] = TRUE;
         }
       }
     }
   }

   // Finding the minimum timestamp of HOL cells (of same VOQ) in K switches
   for(i=0;i<N;i++)
   {
     for(j=0;j<N;j++)
     {
       first_flag = TRUE;
       for(l=0;l<K;l++)
       {
         if((flag1[l][i][j] == TRUE)&&(first_flag == TRUE))
         {
           minimum = temp1[l][i][j];
           channel_num = l;
           first_flag = FALSE;
         }
         else
         {
           if((flag1[l][i][j] == TRUE)&&(temp1[l][i][j]<minimum))
           {
             minimum = temp1[l][i][j];
             channel_num = l;
           }
         }
       }

       // Recording the minimum timestamp and corresponding switch plane number
       if(first_flag == FALSE)
       {
         flag2[i][j] = TRUE;
         temp2[i][j] = minimum;
         temp3[i][j] = channel_num;
       }
     }
   }

   // Infinite Loop
   while(TRUE)
   {
     first_flag = TRUE;
     for(i=0;i<N;i++)
     {
       // If an inport has not been selected yet
       if(flag_inport[i] == FALSE)
       {
         for(j=0;j<N;j++)
         {
           // If an outport has not been selected yet
           if(flag_outport[j] == FALSE)
           {
             // Finding oldest cell in the switch
             if(flag2[i][j] == TRUE)
             {
               if(first_flag == TRUE)
               {
                 minimum = temp2[i][j];
                 min_inport = i;
                 min_outport = j;
                 first_flag = FALSE;
               }
               else
               {
                 if(temp2[i][j]<minimum)
                 {
                   minimum = temp2[i][j];
                   min_inport = i;
                   min_outport = j;
                 }
               }
             }
           }
         }
       }
     }

     // Setting flags corresponding to the selected cell
     if(first_flag == FALSE)
     {
       accept[min_inport][min_outport] = TRUE;
       flag_inport[min_inport] = TRUE;
       flag_outport[min_outport] = TRUE;
     }
     else
     {
       break;
     }
   } 

   // Forwarding selected cells in all K switches
   for(i=0;i<N;i++)
   {
     for(j=0;j<N;j++)
     {
       if(accept[i][j] == TRUE)
       {
         channel_num = temp3[i][j];
         for(l=0;l<K;l++)
         {
           if(queue_cnt(In_Q_event[channel_num][i][j])>0)
           {
             set(In_Q_event[channel_num][i][j]);
             flag1[channel_num][i][j] = FALSE;
           }
           channel_num = (channel_num + 1)%K;
         }
       }
     }
   }

   // Wait for an internal time slot
   hold(K * CELL_TIME);

   // Re-initializations
   for(i=0;i<N;i++)
   {
     for(j=0;j<N;j++)
     {
       accept[i][j] = FALSE;
       flag2[i][j] = FALSE;
     }
   }

   for(i=0;i<N;i++)
   {
     flag_inport[i] = FALSE;
     flag_outport[i] = FALSE;
   }
 }
}
