//======================================================= file = inport.c =====
//=  Input port for Saad PPS switch simulation model                          =
//=============================================================================
//=  Notes: 1) Internal switch planes are CIOQ switch with                    = 	
//=             oldest cell first (OCF) arbitration algorithm                 =
//=---------------------------------------------------------------------------=
//=  Contains: inport()                                                       =
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "math.h"       // Needed for ceil()
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

void inport(int in_id,int out_id,int channel_num,double timestamp,
int tag,double org_time)
{
  double internal_slot; // Intenral time slot 
  double temp;  // Temporary variable

  create("inport");

  // Delaying a cell to the maximum delay (K*N) at demultiplexor
  temp = timestamp + (N*K);
  internal_slot = ceil(temp/K);
  hold((internal_slot*K)-clock);

  // Sending arrival timestamp to OCF arbiter
  send(Tag_number[channel_num][in_id][out_id],(long)timestamp);

  // Queuing cell at internal inport VOQs
  queue(In_Q_event[channel_num][in_id][out_id]);

  // Forward cell to the outport of the same switch plane
  out[channel_num][out_id](in_id, out_id,channel_num,timestamp,
  tag,org_time);

  return;
}
