//======================================================== file = mux.c =======
//=  Outport multiplexor for Saad PPS switch simulation model	            =
//=============================================================================
//=  Contains: mux()                                                          =
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff


void mux(int from_id,int out_id,int channel_num,double timestamp,
int tag,double org_time)
{
  create("mux");

  // If buffer is full, drop the cell 
  if(queue_cnt(Mux_Q_event[channel_num][out_id])==N)
  {
    Mux_loss[out_id]++;
    return;
  }

  // Sending arrival timestamp to mux_sel()
  send(Seq_number[channel_num][out_id],(long)timestamp);

  // Queueat multiplexor buffers
  queue(Mux_Q_event[channel_num][out_id]);

  // Transmitting the cell
  reserve(Outport_external[out_id]);
  hold(CELL_TIME);
  release(Outport_external[out_id]);

  // Record delay statistics for cells leaving this port
  record((clock-org_time),Delay_table[out_id]);
  Outport_count[out_id]++;

  // Recording re-ordering of cells per flow
  if((from_id==0)&&(out_id==0))
  {
    Count++;
    if(tag == Index)
    {
      Cell_served[Index] = TRUE;
      Index++;
      while(Cell_served[Index] == TRUE)
      {
        Index++;
      }
    }
    else
    {
      Cell_served[tag] = TRUE;
      Reorder++;
    }
  }
  
  return;
}
