//====================================================== file = mux_sel.c =====
//=  Mux re-assembly algorithm for Saad PPS  switch simulation model          =
//=============================================================================
//=  Contains: mux_sel()                                                      = 
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

void mux_sel(int out_id)
{
  int i,j;    // Generic loop counters
  int count;  // Counter, to count # of empty buffers in mux      
  long tag;   // Arrival timestamp of HOL cell in mux buffers 
  int flag[K]; // Boolean flags 
  int first_flag,temp_flag;   // Boolean flags 
  int temp1[K];   // Temporary variables
  int minimum;    // Minimum timestamp of a cell among the HOL cells
  int min_layer;  // Switch plane # of a cell (having minimum timestamp)

  create("mux_sel");

  // Initializing the variables
  for(i=0;i<K;i++)
  {
    flag[i] = FALSE;
    temp1[i]= 0;
  }

  // Infinite Loop
  while(TRUE)
  {

    // Hold for synchronization
    hold(0.0);

    // Receving arrival timestamp of HOL cells (from mux buffers)  
    for(i=0;i<K;i++)
    {
      if((flag[i]==FALSE)&&(msg_cnt(Seq_number[i][out_id])>0))
      {
        receive(Seq_number[i][out_id], (long *) &tag);
        temp1[i] = tag;
        flag[i] = TRUE;
      }
    }

    // Checking if all mux buffers are empty 
    count = 0;
    temp_flag=FALSE;

    // Checking for internal outports link status
    for(i=0;i<K;i++)
    {
      if(flag[i] == FALSE)
      {
        count++;
      }
    }

    if(count == K)
      temp_flag = TRUE;

    // Hold for an empty cell slot  
    if(temp_flag == TRUE)
    {
      hold(CELL_TIME);
    }
    else
    {
      // Finding HOL cell with minimum timestamp
      first_flag = TRUE;
      for(i=0;i<K;i++)
      {
        if((first_flag == TRUE)&&(flag[i]==TRUE))
        {
          minimum = temp1[i];
          min_layer = i;
          first_flag = FALSE;
        }
        else
        {
          if((flag[i]==TRUE)&&(temp1[i]<minimum))
          {
            minimum = temp1[i];
            min_layer = i;
          }
        }
      }

      // Transmit the selected cell
      if(first_flag==FALSE)
      {
        set(Mux_Q_event[min_layer][out_id]);
        flag[min_layer]= FALSE;
        hold(CELL_TIME);
      }
 
      // assert if mux_sel() can not find a cell with minimum timestamp 
      else
      {
        assert(FALSE);
      }
    }
  }
}
